'use strict';

var assert = require('assert');
var _ = require('lodash');

function getJsonSchemaType(obj) {
  var type = Object.prototype.toString.call(obj).slice(8, -1);

  switch (type) {
  case 'Number':
    return 'integer';
  case 'String':
    return 'string';
  case 'Boolean':
    return 'boolean';
  case 'Array':
    return 'array';
  case 'Object':
    return 'object';
  default:
    return 'UNKNOWN';
  }
}

function makeNewOption(option, name) {
  var newOption = Object.assign({}, option);
  newOption.name = name;
  return newOption;
}

function getJsonSchema(json, option) {
  var result = {};
  result.id = option.base + (option.name || '');
  result.type = getJsonSchemaType(json);
  switch (result.type) {
  case 'array':
    var newOption = makeNewOption(option, (option.name || '') + '0');
    json.forEach(it => {
      result.items = json2jsonschema(it, newOption);
    });

    result.items = result.items || '';
    break;
  case 'object':
    result.properties = {};
    if (!option.additionalProperties) {
      result.additionalProperties = false;
    }
    for (var key in json) {
      if (json.hasOwnProperty(key)) {
        // option.name = key;
        const newOption = makeNewOption(option, key);
        result.properties[key] = json2jsonschema(json[key], newOption);
      }
    }
    break;
  default:
    break;
  }
  return result;
}

var definitions = {};

function gotDefinition(name, def) {
  var old = definitions[name];
  if (!old) {
    definitions[name] = def;
  } else {
    // var common = _.intersection(Object.keys(old.properties), Object.keys(def.properties));
    // var diff = _.difference(Object.keys(old.properties), Object.keys(def.properties));
    // var result = def;
    // common.forEach(function (key) {
    //   assert.deepEqual(old[key], def[key]);
    // });
    // console.log('DIFF', diff, common);
    // // Object.assign
    // diff.forEach(function (key) {
    //   if (!key in result.properties) {
    //     result.properties[key] = def[key];
    //   };
    // });
    // definitions[name] = result;
    // definitions[name] = Object.assign(def, old);

    _.intersection(Object.keys(old.properties), Object.keys(def.properties)).forEach(key => {
      assert.deepEqual(old[key], def[key]);
    });
    const properties = Object.assign(def.properties, old.properties);
    old.properties = properties;
  }
}

function json2jsonschema(json, option) {
  var name = option.name;
  var result = getJsonSchema(json, option);
  if (name !== null && result.type === 'object') {
    var newResult = {};
    newResult['$ref'] = '#/definitions/' + name;
    gotDefinition(name, result);
    result = newResult;
  }
  return result;
}

function process(json, option) {
  option = option || {};
  option.base = option.base || '';
  if (option.base.length !== 1 && option.base[option.base.length - 1] !== '/') {
    option.base += '/';
  }
  option.name = null;
  if (!'additionalProperties' in option) {
    option.additionalProperties = true;
  }

  definitions = {};
  var result = json2jsonschema(json, option);
  if (Object.keys(definitions).length !== 0) {
    result.definitions = definitions;
  }
  return result;
}

module.exports = process;
